/**
 * @file       servo.c
 * @brief      Servo motor driver
 * 
 * @author     Laurent Saint-Marcel (lstmarcel@yahoo.fr)
 * @date       2004/07/25
 */

#include <io.h>
#include <sig-avr.h>
#include <interrupt.h>

#include "util.h"
#include "servo.h"
#include "uart.h"

/* ------------------------------------------------------------------------ */
/* Local define                                                             */
/* ------------------------------------------------------------------------ */
#define SERVO_MAX_NBR 4

#if SERVO_NBR > SERVO_MAX_NBR
#error SERVO_NBR must be less than SERVO_MAX_NBR
#endif

/* ------------------------------------------------------------------------ */
/* static                                                                   */
/* ------------------------------------------------------------------------ */
static unsigned char servoPosition_[SERVO_MAX_NBR];
static unsigned char servoEnable_=0;
static unsigned char servoId_=0;
static UtilFnPtr servoCB1_=NULL;
static UtilFnPtr servoCB2_=NULL;

/* ------------------------------------------------------------------------ */
/* INTERRUPT(SIG_OVERFLOW0)                                                 */
/* ------------------------------------------------------------------------ */
/* Interrupt when Timer 0 reached overflow                                  */
/* ------------------------------------------------------------------------ */
INTERRUPT(SIG_OVERFLOW0)  
{
	unsigned char id = (servoId_ >> 2);
	switch (servoId_ & 0x03) {
		case 0:
			/* wait 0.5ms at the upper level */
			if (servoEnable_ & (1 << id)) {
				sbi(SERVO_IO_PORT, id + SERVO_FIRST_PIN); 
			}
			outp(230, TCNT0); // 193 = 255-62
			break;
		case 1:
		    /* wait between 0 and 2ms at the upper level */
			outp(servoPosition_[id], TCNT0);
			break;
		case 2:
		    /* wait remaining time at lower level */
			if (id < 6) cbi(SERVO_IO_PORT, id + SERVO_FIRST_PIN); 
			outp(SERVO_POS_MAX - servoPosition_[id], TCNT0);
			servoId_++;
			break;
		default:
			/* should never happen */
			break;
	}
	servoId_++;
	if ((servoId_ >> 2) >= SERVO_MAX_NBR) {
		/* every 10ms */
		servoId_=0;
		if (servoCB1_) servoCB1_();
		if (servoCB2_) servoCB2_();
	}
}

/* ------------------------------------------------------------------------ */
/* servoInit                                                                */
/* ------------------------------------------------------------------------ */
void servoInit()
{
	servoDisableAll();
	/* Set timer 0 speed : CLK/64 = 0.008ms/tick */
	outp(0x03, TCCR0);  
	/* Set timer 0 value, when reach overflow, INTERRUPT(SIG_OVERFLOW0) is run */
	outp(0,TCNT0);
	/* Enable timer 0 interrupt */
	sbi(TIMSK, TOIE0);

	/* Define IO ports as output, lower state */
	unsigned char i=SERVO_FIRST_PIN;
	for(; i < SERVO_FIRST_PIN +SERVO_NBR; i++) {
		sbi(SERVO_IO_DIR, i);
		cbi(SERVO_IO_PORT, i);
	}
	for(i=0; i < SERVO_MAX_NBR; i++) {
		servoPosition_[i]=0;
	}
}

/* ------------------------------------------------------------------------ */
/* servoSetPosition                                                         */
/* ------------------------------------------------------------------------ */
void servoSetPosition(unsigned char servoId,
					  unsigned char position)
{
	if (position > SERVO_POS_MAX) position=SERVO_POS_MAX;
	servoPosition_[servoId] = position;
	servoEnable(servoId);
}

/* ------------------------------------------------------------------------ */
/* servoPosInc                                                              */
/* ------------------------------------------------------------------------ */
unsigned char servoPosInc(unsigned char servoId)
{
	if (servoPosition_[servoId] < SERVO_POS_MAX) {
		servoPosition_[servoId]++;
		servoEnable(servoId);
		return TRUE;
	} else {
		return FALSE;
	}
}

/* ------------------------------------------------------------------------ */
/* servoPosDec                                                              */
/* ------------------------------------------------------------------------ */
unsigned char servoPosDec(unsigned char servoId)
{
	if (servoPosition_[servoId] > SERVO_POS_MIN) {
		servoPosition_[servoId]--;
		servoEnable(servoId);
		return TRUE;
	} else {
		return FALSE;
	}
}


/* ------------------------------------------------------------------------ */
/* servoDisable                                                             */
/* ------------------------------------------------------------------------ */
void servoDisable(unsigned char servoId)
{
	servoEnable_ &= ~(1 << servoId);
}

/* ------------------------------------------------------------------------ */
/* servoDisableAll                                                          */
/* ------------------------------------------------------------------------ */
void servoDisableAll()
{
	servoEnable_ =0;
}

/* ------------------------------------------------------------------------ */
/* servoEnable                                                              */
/* ------------------------------------------------------------------------ */
void servoEnable(unsigned char servoId)
{
	servoEnable_ |= (1 << servoId);
}

/* ------------------------------------------------------------------------ */
/* servoGetPosition                                                         */
/* ------------------------------------------------------------------------ */
unsigned char servoGetPosition(unsigned char servoId)
{
	return servoPosition_[servoId];
}

/* ------------------------------------------------------------------------ */
/* servoRegisterPeriodicCB1                                                  */
/* ------------------------------------------------------------------------ */
void servoRegisterPeriodicCB1(UtilFnPtr callback)
{
	servoCB1_ = callback;
}

/* ------------------------------------------------------------------------ */
/* servoRegisterPeriodicCB2                                                  */
/* ------------------------------------------------------------------------ */
void servoRegisterPeriodicCB2(UtilFnPtr callback)
{
	servoCB2_ = callback;
}

