/**
 * @file       servo.h
 * @brief      Servo motor driver
 * 
 * @author     Laurent Saint-Marcel (lstmarcel@yahoo.fr)
 * @date       2004/07/25
 */

#ifndef __SERVO_H__
#define __SERVO_H__

#include "util.h"

#define SERVO_IO_PORT    PORTB
#define SERVO_IO_DIR     DDRB
#define SERVO_FIRST_PIN  3
#define SERVO_NBR        3

/* Servo Ids */
#define SERVO_WHEEL_LEFT   0 
#define SERVO_WHEEL_RIGHT  1
#define SERVO_SOLAR_DIR    2

/* Servo Positions */
#define SERVO_WHEEL_LEFT_FORWARD    20
#define SERVO_WHEEL_LEFT_BACKWARD  200
#define SERVO_WHEEL_RIGHT_FORWARD  200
#define SERVO_WHEEL_RIGHT_BACKWARD  20

/** @brief Minimum servo position order */
#define SERVO_POS_MIN 0
/** @brief Maximum servo position order */
#define SERVO_POS_MAX 254

/**
 * @brief Initialize the servo command driver and the timer interrupt
 */
void servoInit();

/**
 * @brief Set the position of a servo and enable the asservissement
 * @param servoId id of the servo motor [0..SERVO_NBR[
 * @param position requested position [SERVO_POS_MIN..SERVO_POS_MAX]
 */
void servoSetPosition(unsigned char servoId,
					  unsigned char position);

/**
 * @brief Increment the position of a servo and enable the asservissement
 * @param servoId id of the servo motor [0..SERVO_NBR[
 * @return FALSE if the servo reached its maximum position, TRUE in the other 
 *         case
 */
unsigned char servoPosInc(unsigned char servoId);

/**
 * @brief Decrement the position of a servo and enable the asservissement
 * @param servoId id of the servo motor [0..SERVO_NBR[
 * @return FALSE if the servo reached its minimum position, TRUE in the other 
 *         case
 */
unsigned char servoPosDec(unsigned char servoId);


/**
 * @brief Disable the asservissement of a motor to avoid consuming energy
 * @param servoId id of the servo motor [0..SERVO_NBR[
 */
void servoDisable(unsigned char servoId);

/**
 * @brief Disable the asservissement of all motors to avoid consuming energy
 */
void servoDisableAll();

/**
 * @brief Enable the asservissement of a motor
 * @param servoId id of the servo motor [0..SERVO_NBR[
 * @see servoDisable
 */
void servoEnable(unsigned char servoId);

/**
 * @brief Return the last position order of a servo
 * @param servoId id of the servo motor [0..SERVO_NBR[
 * @return The last position order of a servo
 */
unsigned char servoGetPosition(unsigned char servoId);

/**
 * @brief Register a callback that will be run 150 times per second
 */
void servoRegisterPeriodicCB1(UtilFnPtr callback);

/**
 * @brief Register a callback that will be run 150 times per second
 */
void servoRegisterPeriodicCB2(UtilFnPtr callback);


#endif /* __SERVO_H__ */
