/**
 * @file       solarDir.c
 * @brief      Solar detector that returns the direction of the sun
 * 
 * @author     Laurent Saint-Marcel (lstmarcel@yahoo.fr)
 * @date       2004/08/02
 */

#include "solarDir.h"
#include "uart.h"
#include "adc.h"
#include "servo.h"
#include "pinConfig.h"

/* ------------------------------------------------------------------------ */
/* Local declaration                                                        */
/* ------------------------------------------------------------------------ */
/** @brief None of the detectors is in the shadow */
#define SOLAR_STEP      7

/**
 * @brief Periodic task run after each adc measurement that move the solar 
 *        detector to measure the sun direction
 */
void solarPeriodicTask();

/**
 * @brief Get the ADC value of each solar captor and return the position of the sun
 * @return information concerning how the sun is detected by the captors
 * @see SOLAR_SUN_LOST, SOLAR_SUN_FRONT, SOLAR_SUN_REAR, SOLAR_SUN_INVISBLE
 */
void solarStatus();

void solarCheckCaptors(unsigned char servoPos);

/* ------------------------------------------------------------------------ */
/* static                                                                   */
/* ------------------------------------------------------------------------ */
static unsigned short solarDir_     = SOLAR_DIR_UNKNOWN;
static unsigned char  solarDir1_     = 0x0;
static unsigned char  solarDir2_     = 0x0;
static unsigned char  solarLastDir_  = 0xFF;
static unsigned char  solarMin1_     = 0xFF; // shadow adc level
static unsigned char  solarMax1_     = 0x00; // shadow adc level
static unsigned char  solarMin2_     = 0xFF; // shadow adc level
static unsigned char  solarMax2_     = 0x00; // shadow adc level
static unsigned char  solarMinA_     = 0xFF; // shadow adc level
static unsigned char  solarMaxA_     = 0x00; // shadow adc level

/* ------------------------------------------------------------------------ */
/* solarInit                                                                */
/* ------------------------------------------------------------------------ */
void solarInit()
{
	solarDir_ = SOLAR_DIR_UNKNOWN;
	adcRegisterTreatmentCB(solarPeriodicTask);
	adcAutomaticMeasurement(TRUE);
}

/* ------------------------------------------------------------------------ */
/* solarGetDir                                                              */
/* ------------------------------------------------------------------------ */
unsigned short solarGetDir()
{
	return solarDir_;
}

/* ------------------------------------------------------------------------ */
/* solarCheckCaptors                                                        */
/* ------------------------------------------------------------------------ */
void solarCheckCaptors(unsigned char servoPos)
{
	unsigned char cc=adcGetValue(ADC_SOLAR_CENTER);
	unsigned char ce=adcGetValue(ADC_SOLAR_EXTERN);

	if (cc > solarMax1_) { // shadow
		solarMax1_ = cc;
		solarDir1_ = servoPos;
	} 
	if (cc < solarMin1_) solarMin1_ = cc; // light
	if (ce > solarMax2_) { // shadow
		solarMax2_ = ce;
		solarDir2_ = servoPos;
	} 
	if (ce < solarMin2_) solarMin2_ = ce; // light
}

/* ------------------------------------------------------------------------ */
/* solarStatus                                                              */
/* ------------------------------------------------------------------------ */
void solarStatus()
{
	unsigned char solarDelta;
	if (solarMax1_ > solarMaxA_) solarMaxA_ = solarMax1_;
    if (solarMin1_ < solarMinA_) solarMinA_ = solarMin1_;
	if (solarMax2_ > solarMaxA_) solarMaxA_ = solarMax2_;
	if (solarMin2_ < solarMinA_) solarMinA_ = solarMin2_;
	solarDelta = ((solarMaxA_-solarMinA_) >> 2);
	if ((solarMax2_-solarMin2_) > (solarMax1_-solarMin1_)) {
		// 2 is more in the shadow
		if ((solarMax2_-solarMin2_) > solarDelta) {
			solarDir_ = 0x100+solarDir2_;
		} else {
			solarDir_ |= SOLAR_NOT_VISIBLE;
		}
	} else {
		// 1 is more in the shadow
		if ((solarMax1_-solarMin1_) > solarDelta) {
			solarDir_ = solarDir1_;
		} else {
			solarDir_ |= SOLAR_NOT_VISIBLE;
		}
	}
	solarDump();
	// reset min/max
	solarMin1_     = 0xFF; // shadow adc level
    solarMax1_     = 0x00; // shadow adc level
    solarMin2_     = 0xFF; // shadow adc level
    solarMax2_     = 0x00; // shadow adc level
	solarMinA_     = 0xFF; // shadow adc level
    solarMaxA_     = 0x00; // shadow adc level
}

/* ------------------------------------------------------------------------ */
/* solarDump                                                                */
/* ------------------------------------------------------------------------ */
void solarDump()
{
	uartWrite('O');
	uartWrite(solarMaxA_);
	uartWrite(solarMinA_);
	uartWrite(solarMax1_);
	uartWrite(solarMin1_);
	uartWrite(solarDir1_);
	uartWrite(solarMax2_);
	uartWrite(solarMin2_);
	uartWrite(solarDir2_);
	uartWrite((unsigned char)(solarDir_&0x00FF));
	uartWrite((unsigned char)(solarDir_ >> 8));
}


/* ------------------------------------------------------------------------ */
/* solarPeriodicTask                                                        */
/* ------------------------------------------------------------------------ */
void solarPeriodicTask()
{
	unsigned char servoPos = servoGetPosition(SERVO_SOLAR_DIR);
	solarCheckCaptors(servoPos);
    solarLastDir_+=2;
	if ((solarLastDir_>>1) < SOLAR_STEP) return;
	solarLastDir_ &= 0x01;

	if ((servoPos == SERVO_POS_MIN) && ((solarLastDir_&0x01) == 0)) {
		solarLastDir_ |= 0x01;
	} else if ((servoPos == SERVO_POS_MAX) && ((solarLastDir_&0x01) == 1)){
		solarLastDir_ &= 0xFE;
		// end of a cycle
		solarStatus();
	}
	if (solarLastDir_&0x01) {
		servoPosInc(SERVO_SOLAR_DIR);
	} else {
		servoPosDec(SERVO_SOLAR_DIR);
	}
}


